@@ -52,6 +52,12 @@ def cli(
 @click.option("--tag", required=False, help="Tag of the version to upload")
 @click.option("--verbose", "-v", is_flag=True, default=False, help="Show verbose output")
 @click.option("--job-id", required=False, help="The id of job that is uploading this", default=None)
+@click.option(
+    "--failed-cache",
+    is_flag=True,
+    default=False,
+    help="Upload directory as failed cache archive to job (requires --job-id)",
+)
 @click.pass_context
 def upload(
     ctx: Context,
@@ -62,6 +68,7 @@ def upload(
     tag: str | None,
     verbose: bool,
     job_id: str | None,
+    failed_cache: bool,
 ) -> None:
     try:
         org_name, vault_and_tag = vault_spec.split('/')
@@ -82,9 +89,18 @@ def upload(
         ctx.obj["client"] = client
         if not directory:
             directory = get_foundry_out_dir()
-        response_message = client.upload_files_s3(
-            directory=directory, tag=tag_name or tag, verbose=verbose, job_id=job_id
-        )
+
+        # Validate failed cache requirements first
+        if failed_cache:
+            if not job_id:
+                click.echo("Error: --failed-cache flag requires --job-id to be specified")
+                sys.exit(1)
+
+            response_message = client.upload_failed_cache(directory=directory, job_id=job_id, verbose=verbose)
+        else:  # Normal upload flow
+            response_message = client.upload_files_s3(
+                directory=directory, tag=tag_name or tag, verbose=verbose, job_id=job_id
+            )
         if verbose:
             click.echo(response_message)
     except Exception as e:
