@@ -33,6 +33,7 @@ from .constants import (
     CONFIG_SESSION_PATH,
     DEVICE_LOGIN_URL,
     GRAPHQL_URL,
+    JOB_FILE_UPLOAD_URL,
     KONTROL_JOB_URL,
     ORG_VAULT_CACHE_URLS,
     ORG_VAULT_CACHES_URL,
@@ -146,6 +147,10 @@ class CustomSession(Session):
             logging.error(f"POST request failed: {e}")
             raise e
 
+    def post_text(self, *args: Any, **kwargs: Any) -> Any:
+        response = super().post(*args, **kwargs)
+        return response.text
+
     def get_text(self, *args: Any, **kwargs: Any) -> Any:
         response = super().get(*args, **kwargs)
         return response.text
@@ -292,6 +297,11 @@ class KaasClient:
         )
         return data
 
+    def _get_job_file_upload_url(self, job_id: str, file_name: str) -> str:
+        """Get presigned URL for uploading a file to a job."""
+        data = self._session.post_text(url=f'{self._url}{JOB_FILE_UPLOAD_URL.format(job_id, file_name)}')
+        return data
+
     def _upload_file(
         self,
         file_path: Path,
@@ -691,6 +701,65 @@ class KaasClient:
 
         return None
 
+    def upload_failed_cache(self, directory: str, job_id: str, verbose: bool) -> str | None:
+        """Upload a failed cache directory as a zip file to the specified job."""
+        click.echo(f"Uploading failed cache from directory: {directory}")
+
+        # Check if directory exists
+        if not Path(directory).exists():
+            raise KaasCliException(f'Directory does not exist: {directory}')
+
+        # Create temp directory
+        temp_dir = Path(tempfile.mkdtemp())
+        if verbose:
+            click.echo(f"Temp directory created at {temp_dir}")
+
+        try:
+            # Get all files in the directory
+            file_list = self._list_local_files(directory)
+            if not file_list or len(file_list) == 0:
+                raise KaasCliException(f'No files to upload in dir: {directory}')
+
+            # Create zip archive of the entire directory
+            zip_filename = "failed_cache.zip"
+            archive_path = self.archive_files(
+                file_list, zip_filename, archive_format='zip', root_dir=Path(directory), target_dir=temp_dir
+            )
+
+            if verbose:
+                click.echo(f"Created archive: {archive_path}")
+
+            # Get presigned URL for job file upload
+            upload_url = self._get_job_file_upload_url(job_id, zip_filename)
+            if not upload_url:
+                raise KaasCliException(f"Failed to get upload URL for job {job_id}")
+
+            # Upload the zip file
+            file_size = Path(archive_path).stat().st_size
+            with open(archive_path, 'rb') as file:
+                if verbose:
+                    click.echo(f"Uploading {archive_path} to job {job_id}")
+
+                headers = {'Content-Length': str(file_size)}
+                response = requests.put(upload_url, data=file, headers=headers)
+
+                if response.status_code in [200, 201]:
+                    if verbose:
+                        click.echo(f"Successfully uploaded failed cache to job {job_id}")
+                    return f"Failed cache uploaded successfully to job {job_id}"
+                else:
+                    raise KaasCliException(
+                        f"Failed to upload failed cache. Status code: {response.status_code}, "
+                        f"Response: {response.text}"
+                    )
+        except Exception as e:
+            raise KaasCliException(f"Failed to upload failed cache: {e}") from None
+        finally:
+            # Clean up temp directory
+            shutil.rmtree(temp_dir)
+            if verbose:
+                click.echo(f"Temp directory {temp_dir} removed")
+
     def download_version_or_tag(
         self, org_name: str, vault_name: str, version_or_tag: str, target_directory: str, verbose: bool
     ) -> Any:
